<?php
/**
 * @version     1.0.0
 * @package     com_docebo
 * @copyright   Copyright (C) 2013. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Docebo Srl. <info@docebo.com> - http://www.docebo.com
 */

// no direct access
defined('_JEXEC') or die;

JHtml::_('behavior.tooltip');
JHtml::_('behavior.formvalidation');
JHtml::_('behavior.keepalive');
// Import CSS
$document = JFactory::getDocument();
$document->addStyleSheet('components/com_docebo/assets/css/docebo.css');
$document->addScript('components/com_docebo/assets/js/docebo.js');

//------------------------------------------------------------------------------------
// Docebo API Class
global $url, $key, $sso, $secret_key, $getset;

class Api extends JViewLegacy {
	static public $totalmembers;

	static public function getSet() {
		$db      = JFactory::getDbo();
		$tPrefix = $db->getPrefix() . 'docebo_general_settings';
		$db->setQuery("SELECT * FROM " . $tPrefix . " Limit 1");
		$general_settings = $db->loadAssoc();

		return $general_settings;
	}

	static public function getHash($params) {
		$getset     = self::getSet();
		$key        = $getset['api_key'];
		$secret_key = $getset['key'];

		$res           = array('sha1' => '', 'x_auth' => '');
		$res['sha1']   = sha1(implode(',', $params) . ',' . $secret_key);
		$res['x_auth'] = base64_encode($key . ':' . $res['sha1']);

		return $res;
	}

	static private function getDefaultHeader($x_auth) {
		$getset = self::getSet();
		$url    = $getset['cloud_address'];

		// Get the host/domain from the cloud URL
		$url = str_replace('https://', '', $url);
		$url = str_replace('http://', '', $url);
		$url = parse_url('http://' . $url, PHP_URL_HOST);


		return array("Host: " . $url, "Content-Type: multipart/form-data", 'X-Authorization: Docebo ' . $x_auth);
	}

	static public function call($action, $data_params) {
		$getset = self::getSet();
		$url    = trim($getset['cloud_address']);

		$url = str_ireplace('http://', '', $url);
		$url = str_ireplace('https://', '', $url);

		$key        = $getset['api_key'];
		$secret_key = $getset['key'];
		$sso        = $getset['sso'];
		if (!function_exists('curl_version')) {
			return FALSE;
		}
		$curl        = curl_init();
		$hash_info   = self::getHash($data_params);
		$http_header = self::getDefaultHeader($hash_info['x_auth']);
		$opt         = array(
			CURLOPT_URL            => $url . '/api/' . $action,
			CURLOPT_RETURNTRANSFER => 1,
			CURLOPT_HTTPHEADER     => $http_header,
			CURLOPT_POST           => 1,
			CURLOPT_POSTFIELDS     => $data_params,
			CURLOPT_CONNECTTIMEOUT => 5, // Timeout to 5 seconds
			CURLOPT_SSL_VERIFYPEER => FALSE,
			CURLOPT_SSL_VERIFYHOST => FALSE,
		);
		curl_setopt_array($curl, $opt);
		// $output contains the output string
		$output = curl_exec($curl);

		// it closes the session
		curl_close($curl);

		return $output;
	}

	static public function sso($user) {
		$getset = self::getSet();
		$url    = trim($getset['cloud_address']);
		if (!$url)
			return FALSE;

		if (stripos($url, 'http://') === FALSE && stripos($url, 'https://') === FALSE)
			$url = 'http://' . $url;
		$sso = $getset['sso'];

		// Get an instance of the cache object
		$cache = JFactory::getCache();

		// Get the current user's ID so we can get his Docebo username through API
		$user = JFactory::getUser()->get('id');

		$username = $cache->call(array('Api', 'userToDoceboUser'), $user);

		$time  = time();
		$token = md5(strtolower($username) . ',' . $time . ',' . $sso);

		return $url . '/doceboLms/index.php?auth_regen=1&modname=login&op=confirm&login_user=' . urlencode(strtolower($username)) . '&time=' . $time . '&token=' . $token;
	}

	static public function userToDoceboUser($user) {

		// Get the Docebo username that is associated
		// with the currently logged in Joomla user
		// (through API)
		$params           = array(
			'ext_user'      => $user,
			'ext_user_type' => 'wordpress',
		);
		$apiResponse      = self::call('user/profile/', $params);
		$docebo_user_info = json_decode($apiResponse);

		if ($docebo_user_info && isset($docebo_user_info->userid))
			return $docebo_user_info->userid;

		$user = JFactory::getUser($user);
		if($user){
			return $user->username;
		}

		return FALSE;
	}
}


?>
<script src="//ajax.googleapis.com/ajax/libs/jquery/1.8.3/jquery.min.js"></script>
<script type="text/javascript">


	Joomla.submitbutton = function (task) {

		if (task == 'generalsetting.cancel' || document.formvalidator.isValid(document.id('generalsetting-form'))) {
			Joomla.submitform(task, document.getElementById('generalsetting-form'));
		}
		else {
			alert('<?php echo $this->escape(JText::_('JGLOBAL_VALIDATION_FORM_FAILED'));?>');
		}
	}
</script>
<style>#jform_scenario label {
		width: 96%;
	}</style>
<form action="<?php echo JRoute::_('index.php?option=com_docebo&layout=edit&id=' . (int) $this->item->id); ?>"
      method="post" enctype="multipart/form-data" name="adminForm" id="generalsetting-form" class="form-validate">
	<div class="width-60 fltlft">
		<fieldset class="adminform">
			<legend><?php echo JText::_('COM_DOCEBO_LEGEND_GENERALSETTING'); ?></legend>
			<ul class="adminformlist">

				<input type="hidden" name="jform[id]" value="<?php echo $this->item->id; ?>"/>
				<li><?php echo $this->form->getLabel('scenario'); ?>
					<?php echo $this->form->getInput('scenario'); ?></li>
				<li><?php echo $this->form->getLabel('cloud_address'); ?>
					<?php echo $this->form->getInput('cloud_address'); ?></li>
				<li><?php echo $this->form->getLabel('api_key'); ?>
					<?php echo $this->form->getInput('api_key'); ?></li>
				<li><?php echo $this->form->getLabel('key'); ?>
					<?php echo $this->form->getInput('key'); ?></li>
				<li><?php echo $this->form->getLabel('sso'); ?>
					<?php echo $this->form->getInput('sso'); ?></li>


			</ul>

			<p style="clear:both">&nbsp;</p>
			<?php
			$testApiCall = json_decode(Api::call('user/apiResponseTest', array()), TRUE);
			$apiStatus = FALSE;

			if (isset($testApiCall['api_status']) && $testApiCall['api_status'] == 'available')
				$apiStatus = TRUE;

			// We should call an old API method for backward
			// compatibility in case the new method is not yet
			// present in the currently called Docebo LMS.
			if (!$apiStatus) {
				$testApiCall = json_decode(Api::call('user/count', array()), TRUE);
				if (isset($testApiCall['count']))
					$apiStatus = TRUE;
			}
			?>
			<p class="api-<?php echo ($apiStatus) ? 'on' : 'off'; ?>"><?php echo ($apiStatus) ? JText::_('COM_DOCEBO_API_ON') : JText::_('COM_DOCEBO_API_OFF'); ?></p>
		</fieldset>
	</div>


	<input type="hidden" name="task" value=""/>
	<?php echo JHtml::_('form.token'); ?>
	<div class="clr"></div>

	<style type="text/css">
		/* Temporary fix for drifting editor fields */
		.adminformlist li {
			clear: both;
		}
	</style>
</form>