<?php
/**
 * @version     1.0.0
 * @package     com_docebo
 * @copyright   Copyright (C) 2013. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Docebo Srl. <info@docebo.com> - http://www.docebo.com
 */

// no direct access
defined('_JEXEC') or die;

jimport('joomla.plugin.plugin');

/**
 * User - Docebo Plugin
 *
 * @package        Joomla.Plugin
 * @subpakage    Docebo.Docebo
 */
class plgUserDocebo extends JPlugin {

	/**
	 * Constructor.
	 *
	 * @param          $subject
	 * @param    array $config
	 */

	/*function __construct(&$subject, $config = array()) {
		// call parent constructor
		parent::__construct($subject, $config);
	}
	*/


	public function onUserAfterSave($data, $isNew, $result, $error) {


		$userId = JArrayHelper::getValue($data, 'id', 0, 'int');

		$syncedUsers      = 0;
		$failedUsers      = 0;
		$sync             = FALSE;
		$db               = JFactory::getDbo();
		$tPrefix          = $db->getPrefix() . 'users';
		$usersToPassToApi = array();
		$i                = 0;
		$apiCall          = NULL;

		// Prepare the array of users to sync
		// (just one in this case, the one being created/updated)
		if ($isNew) {
			// Pass the data provided in the registration
			// form for user creation through LMS api
			$usersToPassToApi = array($data);

			$userdata = array();

			foreach ($usersToPassToApi as $user) {
				if ($user['id'] > 0) {
					if (strpos($user['name'], ' ') == FALSE) {
						$fname = $user['name'];
						$lname = '';
					} else {
						$name  = explode(' ', $user['name']);
						$fname = $name[0];
						$lname = $name[1];
					}


					$tmp = array('ext_user'      => trim($user['id']),
					             'ext_user_type' => 'wordpress',
					             'wp_user_id'    => $user['username'],
					             'wp_username'   => $user['username'],
					             'userid'        => mb_convert_encoding($user['username'], 'UTF-8', mb_detect_encoding($user['username'], 'auto')), //$user['username'],
					             'email'         => $user['email'],
					             'first_name'    => $fname,
					             'last_name'     => $lname);

					if ($user['password_clear']) {
						// For now we don't pass the cleartext password
						// even if its available
						// $tmp['password'] = $user['password_clear'];
					}
					$userdata[] = $tmp;

					$i++;
				}
			}

			if (!empty($userdata)) {
				$sendData = array('users' => urlencode(json_encode($userdata)));

				$apiCall = Api::call('user/syncWordpressUsers/', $sendData);
			}
		} else {
			// Updating an existing user

			if ($userId > 0) {
				if (strpos($data['name'], ' ') == FALSE) {
					$fname = $data['name'];
					$lname = '';
				} else {
					$name  = explode(' ', $data['name']);
					$fname = $name[0];
					$lname = $name[1];
				}


				$tmp = array('ext_user'      => $userId,
				             'ext_user_type' => 'wordpress',
				             'wp_user_id'    => $data['username'],
				             'wp_username'   => $data['username'],
				             'userid'        => $data['username'],
				             'email'         => $data['email'],
				             'first_name'    => $fname,
				             'last_name'     => $lname,
				             'single_user'   => 'true',
				             'valid'         => '1'
				);

				if ($data['password_clear']) {
					// For now we don't pass the cleartext password
					// even if its available
					// $tmp['password'] = $data['password_clear'];
				}

				foreach($tmp as &$elem){
					$elem = urlencode($elem);
				}

				// Call the API to update the existing user
				$apiCall = Api::call('user/edit/', $tmp);

			}
		}

		$response = json_decode($apiCall, TRUE);

		if (isset($response['success']) && $response['success'] == TRUE) {
			// API call successful

			if (isset($response['result']) && is_array($response['result'])) {
				foreach ($response['result'] as $singleUser) {
					if ($singleUser['success'] == TRUE) {
						$syncedUsers++;
					} else {
						$failedUsers++;
					}
				}
			}
		}

		$sync = '<p>Your users are now synchronized. (Success: ' . $syncedUsers . ', Failure: ' . $failedUsers . ')</p>';

	}


}


//------------------------------------------------------------------------------------
// Docebo API Class
global $url, $key, $sso, $secret_key, $getset;

class Api extends JViewLegacy {
	static public $totalmembers;

	static public function getSet() {
		$db      = JFactory::getDbo();
		$tPrefix = $db->getPrefix() . 'docebo_general_settings';
		$db->setQuery("SELECT * FROM " . $tPrefix . " Limit 1");
		$general_settings = $db->loadAssoc();

		return $general_settings;
	}

	static public function getHash($params) {
		$getset     = self::getSet();
		$key        = $getset['api_key'];
		$secret_key = $getset['key'];

		$res           = array('sha1' => '', 'x_auth' => '');
		$res['sha1']   = sha1(implode(',', $params) . ',' . $secret_key);
		$res['x_auth'] = base64_encode($key . ':' . $res['sha1']);

		return $res;
	}

	static private function getDefaultHeader($x_auth) {
		$getset = self::getSet();
		$url    = $getset['cloud_address'];

		// Get the host/domain from the cloud URL
		$url = str_replace('https://', '', $url);
		$url = str_replace('http://', '', $url);
		$url = parse_url('http://' . $url, PHP_URL_HOST);

		return array("Host: " . $url, "Content-Type: multipart/form-data", 'X-Authorization: Docebo ' . $x_auth);
	}

	static public function check_sync_scene() {
		$sync_scene = '';
		$db         = JFactory::getDbo();
		$tPrefix    = $db->getPrefix() . 'docebo_general_settings';
		$db->setQuery("SELECT scenario FROM " . $tPrefix . " Limit 1");
		$sync_scene = $db->loadResult();

		return $sync_scene;
	}


	static public function call($action, $data_params) {

		$sync_scene = self::check_sync_scene();

		if ($sync_scene == 0) {
			$getset = self::getSet();
			$url    = trim($getset['cloud_address']);
			if (!$url)
				return FALSE;

			$url = str_ireplace('http://', '', $url);
			$url = str_ireplace('https://', '', $url);

			$key        = $getset['api_key'];
			$secret_key = $getset['key'];
			$sso        = $getset['sso'];
			if (!function_exists('curl_version')) {
				return FALSE;
			}
			$curl        = curl_init();
			$hash_info   = self::getHash($data_params);
			$http_header = self::getDefaultHeader($hash_info['x_auth']);
			$opt         = array(
				CURLOPT_URL            => $url . '/api/' . $action,
				CURLOPT_RETURNTRANSFER => 1,
				CURLOPT_HTTPHEADER     => $http_header,
				CURLOPT_POST           => 1,
				CURLOPT_POSTFIELDS     => $data_params,
				CURLOPT_CONNECTTIMEOUT => 5, // Timeout to 5 seconds
				CURLOPT_SSL_VERIFYPEER => FALSE,
				CURLOPT_SSL_VERIFYHOST => FALSE,
			);
			curl_setopt_array($curl, $opt);
			// $output contains the output string
			$output = curl_exec($curl);
			// it closes the session
			curl_close($curl);

			return $output;
		}

	}

	static public function sso($user) {
		// The current Joomla user object
		// (see http://docs.joomla.org/Accessing_the_current_user_object)
		$joomlaUser = JFactory::getUser();
		$user       = $joomlaUser->id;

		$getset = self::getSet();
		$url    = trim($getset['cloud_address']);
		if (!$url)
			return FALSE;

		if (stripos($url, 'http://') === FALSE && stripos($url, 'https://') === FALSE)
			$url = 'http://' . $url;

		$sso = $getset['sso'];

		$time  = time();
		$token = md5(strtolower($user) . ',' . $time . ',' . $sso);

		return $url . '/doceboLms/index.php?auth_regen=1&modname=login&op=confirm&login_user=' . urlencode(strtolower($user)) . '&time=' . $time . '&token=' . $token;
	}


}

?>